@@ -6,12 +6,14 @@ registered by `register_verb` and should be placed in `./verbs.py`
 import itertools
 
 import numpy as np
-import pandas
+import pandas as pd
 from pandas.api.types import is_scalar
+from pandas.core.groupby import SeriesGroupBy
 from pipda import register_func
 
 from ..core.middlewares import WithDataEnv
 from ..core.contexts import Context
+from ..core.factory import func_factory
 from ..core.tibble import Tibble
 from ..core.utils import arg_match, name_of
 from ..core.names import repair_names
@@ -55,7 +57,7 @@ def cut(
     if labels is None:
         ordered_result = True
 
-    return pandas.cut(
+    return pd.cut(
         x,
         breaks,
         labels=labels,
@@ -66,6 +68,56 @@ def cut(
     )
 
 
+@func_factory("apply", "x")
+def diff(x, lag: int = 1, differences: int = 1):
+    """Calculates suitably lagged and iterated differences.
+
+    If the data is a vector of length n and differences = 1, then the computed
+    result is equal to the successive differences
+    `x[lag:] – x[:-lag]`.
+
+    Examples:
+        >>> rv = [52, 21, 10, 11, 19]
+        >>> data = diff(rv)
+        >>> # -31 -11 1 8
+        >>> # rv[1:] - rv[:-1]
+        >>> # rv[1:]  [21, 10, 11, 19]
+        >>> # rv[:-1] [52, 21, 10, 11]
+
+    Args:
+        x: The data
+        lag: The lag to use. Could be negative.
+            It always calculates `x[lag:] - x[:-lag]` even when `lag` is negative
+        differences: The order of the difference
+
+    Returns:
+        An array of `x[lag:] – x[:-lag]`.
+        If `differences > 1`, the rule applies `differences` times on `x`
+    """
+    x = x.values
+    if lag * differences >= x.size:
+        return np.array([], dtype=x.dtype)
+
+    for _ in range(differences):
+        x = x[lag:] - x[:-lag]
+    return x
+
+
+def _diff_sgb_post(out, x, lag=1, differences=1):
+    """Post process diff on SeriesGroupBy object"""
+    non_na_out = out[out.transform(len) > 0]
+    non_na_out = non_na_out.explode()
+    grouping = pd.Categorical(non_na_out.index, categories=out.index.unique())
+    return (
+        non_na_out.explode()
+        .reset_index(drop=True)
+        .groupby(grouping, observed=False)
+    )
+
+
+diff.register(SeriesGroupBy, func=None, post=_diff_sgb_post)
+
+
 @register_func(None, context=Context.EVAL)
 def identity(x):
     """Return whatever passed in
