@@ -1,10 +1,10 @@
 """Functions related to randomization"""
 import random as std_random
 
-import numpy
+import numpy as np
 
 
-def set_seed(seed: int) -> None:
+def set_seed(seed):
     """Set the seed for randomization"""
     std_random.seed(seed)
-    numpy.random.seed(seed)
+    np.random.seed(seed)
