@@ -2,24 +2,19 @@
 
 See source https://github.com/tidyverse/dplyr/blob/master/R/arrange.R
 """
-from typing import Any
 from pandas import DataFrame
 from pipda import register_verb
 
 from ..core.contexts import Context
-from ..core.utils import check_column_uniqueness, reconstruct_tibble
+from ..core.utils import regcall
+from ..core.tibble import TibbleGrouped
+from ..core.exceptions import NameNonUniqueError
 from ..base import union
-from .group_data import group_vars
-from .group_by import ungroup
 from .mutate import mutate
 
-# pylint: disable=no-value-for-parameter
-
 
 @register_verb(DataFrame, context=Context.PENDING)
-def arrange(
-    _data: DataFrame, *args: Any, _by_group: bool = False, **kwargs: Any
-) -> DataFrame:
+def arrange(_data, *args, _by_group=False, **kwargs):
     """orders the rows of a data frame by the values of selected columns.
 
     The original API:
@@ -41,22 +36,27 @@ def arrange(
             Groups are not modified.
             Data frame attributes are preserved.
     """
-    if not args and not kwargs:
-        return _data
+    if not args and not kwargs and not _by_group:
+        return _data.copy()
 
-    check_column_uniqueness(
-        _data, "Cannot arrange a data frame with duplicate names"
-    )
+    if not _data.columns.is_unique:
+        raise NameNonUniqueError(
+            "Cannot arrange a data frame with duplicate names."
+        )
 
-    if not _by_group:
-        sorting_df = mutate(ungroup(_data), *args, **kwargs, _keep="none")
-        sorting_df = sorting_df.sort_values(by=sorting_df.columns.tolist())
+    gvars = getattr(_data, "group_vars", [])
+
+    sorting_df = regcall(mutate, _data, *args, **kwargs)
+    if _by_group:
+        sorting_cols = regcall(union, gvars, sorting_df._datar["mutated_cols"])
     else:
-        gvars = group_vars(_data)
-        sorting_df = ungroup(mutate(_data, *args, **kwargs, _keep="none"))
-        by = union(gvars, sorting_df.columns)
-        sorting_df = sorting_df.sort_values(by=by)
+        sorting_cols = sorting_df._datar["mutated_cols"]
 
-    out = _data.loc[sorting_df.index, :].reset_index(drop=True)
+    sorting_df = DataFrame(sorting_df, copy=False).sort_values(
+        list(sorting_cols), na_position="last"
+    )
+    out = _data.reindex(sorting_df.index)
+    if isinstance(_data, TibbleGrouped):
+        out.reset_index(drop=True, inplace=True)
 
-    return reconstruct_tibble(_data, out, keep_rowwise=True)
+    return out
