@@ -1,38 +1,39 @@
 """Mutating joins"""
-from typing import Iterable, Mapping, Union
-
-import pandas
-from pandas import DataFrame, Series, Categorical
-from pandas.core.dtypes.common import is_categorical_dtype
+import pandas as pd
+from pandas import Categorical, DataFrame
+from pandas.api.types import is_scalar, is_categorical_dtype
 from pipda import register_verb
 
+
 from ..core.contexts import Context
-from ..core.types import StringOrIter, is_scalar
-from ..core.grouped import DataFrameGroupBy
-from ..core.utils import reconstruct_tibble
+from ..core.tibble import reconstruct_tibble
+from ..core.utils import regcall
 from ..base import intersect, setdiff, union
-from .group_by import group_by_drop_default
-from .group_data import group_data, group_vars
 from .dfilter import filter as filter_
 
 
 def _join(
-    x: DataFrame,
-    y: DataFrame,
-    how: str,
-    by: Union[StringOrIter, Mapping[str, str]] = None,
-    copy: bool = False,
-    suffix: Iterable[str] = ("_x", "_y"),
-    # na_matches: str = "", # TODO: how?
-    keep: bool = False,
-) -> DataFrame:
+    x,
+    y,
+    how,
+    by=None,
+    copy=False,
+    suffix=("_x", "_y"),
+    # na_matches = "", # TODO: how?
+    keep=False,
+):
     """General join"""
+    # make sure df.x returns a Series not SeriesGroupBy for TibbleGrouped
+    newx = DataFrame(x, copy=False)
+    y = DataFrame(y, copy=False)
+
     if by is not None and not by:
-        ret = pandas.merge(x, y, how="cross", copy=copy, suffixes=suffix)
+        ret = pd.merge(newx, y, how="cross", copy=copy, suffixes=suffix)
+
     elif isinstance(by, dict):
         right_on = list(by.values())
-        ret = pandas.merge(
-            x,
+        ret = pd.merge(
+            newx,
             y,
             left_on=list(by.keys()),
             right_on=right_on,
@@ -42,16 +43,17 @@ def _join(
         )
         if not keep:
             ret.drop(columns=right_on, inplace=True)
+
     elif keep:
         if by is None:
-            by = intersect(x.columns, y.columns)
+            by = regcall(intersect, newx.columns, y.columns)
         # on=... doesn't keep both by columns in left and right
         left_on = [f"{col}{suffix[0]}" for col in by]
         right_on = [f"{col}{suffix[1]}" for col in by]
-        x = x.rename(columns=dict(zip(by, left_on)))
+        newx = newx.rename(columns=dict(zip(by, left_on)))
         y = y.rename(columns=dict(zip(by, right_on)))
-        ret = pandas.merge(
-            x,
+        ret = pd.merge(
+            newx,
             y,
             left_on=left_on,
             right_on=right_on,
@@ -59,21 +61,26 @@ def _join(
             copy=copy,
             suffixes=suffix,
         )
+
     else:
         if by is None:
-            by = intersect(x.columns, y.columns)
-        by = [by] if is_scalar(by) else by  # type: ignore
-        ret = pandas.merge(x, y, on=by, how=how, copy=copy, suffixes=suffix)
+            by = regcall(intersect, newx.columns, y.columns)
+
+        by = [by] if is_scalar(by) else list(by)
+        ret = pd.merge(newx, y, on=by, how=how, copy=copy, suffixes=suffix)
         for col in by:
+            # try recovering factor columns
             if is_categorical_dtype(x[col]) and is_categorical_dtype(y[col]):
                 ret[col] = Categorical(
                     ret[col],
-                    categories=union(
-                        x[col].cat.categories, y[col].cat.categories
+                    categories=regcall(
+                        union,
+                        x[col].cat.categories,
+                        y[col].cat.categories,
                     ),
                 )
 
-    return reconstruct_tibble(x, ret, keep_rowwise=True)
+    return reconstruct_tibble(x, ret)
 
 
 @register_verb(
@@ -82,18 +89,20 @@ def _join(
     extra_contexts={"by": Context.SELECT},
 )
 def inner_join(
-    x: DataFrame,
-    y: DataFrame,
-    by: Union[StringOrIter, Mapping[str, str]] = None,
-    copy: bool = False,
-    suffix: Iterable[str] = ("_x", "_y"),
-    keep: bool = False,
-) -> DataFrame:
+    x,
+    y,
+    by=None,
+    copy=False,
+    suffix=("_x", "_y"),
+    keep=False,
+):
     """Mutating joins including all rows in x and y.
 
     Args:
         x, y: A pair of data frames
         by: A character vector of variables to join by.
+            If keys from `x` and `y` are different, use a dict
+            (i.e. `{"colX": "colY"}`) instead of a list.
         copy: If x and y are not from the same data source, and copy is
             TRUE, then y will be copied into the same src as x.
             This allows you to join tables across srcs, but it is a
@@ -123,13 +132,13 @@ def inner_join(
     extra_contexts={"by": Context.SELECT},
 )
 def left_join(
-    x: DataFrame,
-    y: DataFrame,
-    by: Union[StringOrIter, Mapping[str, str]] = None,
-    copy: bool = False,
-    suffix: Iterable[str] = ("_x", "_y"),
-    keep: bool = False,
-) -> DataFrame:
+    x,
+    y,
+    by=None,
+    copy=False,
+    suffix=("_x", "_y"),
+    keep=False,
+):
     """Mutating joins including all rows in x.
 
     See Also:
@@ -152,13 +161,13 @@ def left_join(
     extra_contexts={"by": Context.SELECT},
 )
 def right_join(
-    x: DataFrame,
-    y: DataFrame,
-    by: Union[StringOrIter, Mapping[str, str]] = None,
-    copy: bool = False,
-    suffix: Iterable[str] = ("_x", "_y"),
-    keep: bool = False,
-) -> DataFrame:
+    x,
+    y,
+    by=None,
+    copy=False,
+    suffix=("_x", "_y"),
+    keep=False,
+):
     """Mutating joins including all rows in y.
 
     See Also:
@@ -185,13 +194,13 @@ def right_join(
     extra_contexts={"by": Context.SELECT},
 )
 def full_join(
-    x: DataFrame,
-    y: DataFrame,
-    by: Union[StringOrIter, Mapping[str, str]] = None,
-    copy: bool = False,
-    suffix: Iterable[str] = ("_x", "_y"),
-    keep: bool = False,
-) -> DataFrame:
+    x,
+    y,
+    by=None,
+    copy=False,
+    suffix=("_x", "_y"),
+    keep=False,
+):
     """Mutating joins including all rows in x or y.
 
     See Also:
@@ -214,27 +223,30 @@ def full_join(
     extra_contexts={"by": Context.SELECT},
 )
 def semi_join(
-    x: DataFrame,
-    y: DataFrame,
-    by: Union[StringOrIter, Mapping[str, str]] = None,
-    copy: bool = False,
-) -> DataFrame:
+    x,
+    y,
+    by=None,
+    copy=False,
+):
     """Returns all rows from x with a match in y.
 
     See Also:
         [`inner_join()`](datar.dplyr.join.inner_join)
     """
-    ret = pandas.merge(
-        x,
-        y,
+    on = _merge_on(by)
+    right_on = on.get("right_on", on.get("on", y.columns))
+
+    ret = pd.merge(
+        DataFrame(x, copy=False),
+        # fix #71: semi_join returns duplicated rows
+        DataFrame(y, copy=False).drop_duplicates(right_on),
         how="left",
         copy=copy,
         suffixes=["", "_y"],
         indicator="__merge__",
-        **_merge_on(by)
+        **on,
     )
-    ret = ret.loc[ret["__merge__"] == "both", x.columns.tolist()]
-
+    ret = ret.loc[ret["__merge__"] == "both", x.columns]
     return reconstruct_tibble(x, ret)
 
 
@@ -244,27 +256,26 @@ def semi_join(
     extra_contexts={"by": Context.SELECT},
 )
 def anti_join(
-    x: DataFrame,
-    y: DataFrame,
-    by: Union[StringOrIter, Mapping[str, str]] = None,
-    copy: bool = False,
-) -> DataFrame:
+    x,
+    y,
+    by=None,
+    copy=False,
+):
     """Returns all rows from x without a match in y.
 
     See Also:
         [`inner_join()`](datar.dplyr.join.inner_join)
     """
-    ret = pandas.merge(
-        x,
-        y,
+    ret = pd.merge(
+        DataFrame(x, copy=False),
+        DataFrame(y, copy=False),
         how="left",
         copy=copy,
         suffixes=["", "_y"],
         indicator=True,
-        **_merge_on(by)
+        **_merge_on(by),
     )
-    ret = ret.loc[ret._merge != "both", x.columns.tolist()]
-
+    ret = ret.loc[ret._merge != "both", x.columns]
     return reconstruct_tibble(x, ret)
 
 
@@ -274,13 +285,13 @@ def anti_join(
     extra_contexts={"by": Context.SELECT},
 )
 def nest_join(
-    x: DataFrame,
-    y: DataFrame,
-    by: Union[StringOrIter, Mapping[str, str]] = None,
-    copy: bool = False,
-    keep: bool = False,
-    name: str = None,
-) -> DataFrame:
+    x,
+    y,
+    by=None,
+    copy=False,
+    keep=False,
+    name=None,
+):
     """Returns all rows and columns in x with a new nested-df column that
     contains all matches from y
 
@@ -288,56 +299,46 @@ def nest_join(
         [`inner_join()`](datar.dplyr.join.inner_join)
     """
     on = by
+    newx = DataFrame(x, copy=False)
+    y = DataFrame(y, copy=False)
     if isinstance(by, (list, tuple, set)):
         on = dict(zip(by, by))
     elif by is None:
-        common_cols = intersect(x.columns.tolist(), y.columns)
+        common_cols = regcall(intersect, newx.columns, y.columns)
         on = dict(zip(common_cols, common_cols))
     elif not isinstance(by, dict):
-        on = {by: by}  # type: ignore
+        on = {by: by}
 
     if copy:
-        x = x.copy()
+        newx = newx.copy()
 
-    def get_nested_df(row: Series) -> DataFrame:
+    def get_nested_df(row):
+        row = getattr(row, "obj", row)
         condition = None
         for key in on:
             if condition is None:
                 condition = y[on[key]] == row[key]
             else:
                 condition = condition & (y[on[key]] == row[key])
-        df = filter_(y, condition)
+        df = regcall(filter_, y, condition)
         if not keep:
-            df = df[setdiff(df.columns, on.values())]
+            df = df[regcall(setdiff, df.columns, list(on.values()))]
 
         return df
 
-    y_matched = x.apply(get_nested_df, axis=1)
-    y_name = name or getattr(y, "__dfname__", None)
+    y_matched = newx.apply(get_nested_df, axis=1)
+    y_name = name or "_y_joined"
     if y_name:
         y_matched = y_matched.to_frame(name=y_name)
 
-    out = pandas.concat([x, y_matched], axis=1)
-
-    if isinstance(x, DataFrameGroupBy):
-        return x.__class__(
-            x,
-            _group_vars=group_vars(x),
-            _group_drop=group_by_drop_default(x),
-            _group_data=group_data(x),
-        )
-    return out
+    out = pd.concat([newx, y_matched], axis=1)
+    return reconstruct_tibble(x, out)
 
 
-def _merge_on(
-    by: Union[StringOrIter, Mapping[str, str]]
-) -> Mapping[str, StringOrIter]:
+def _merge_on(by):
     """Calculate argument on for pandas.merge()"""
     if by is None:
         return {}
     if isinstance(by, dict):
-        return {
-            'left_on': list(by),
-            'right_on': list(by.values())
-        }
-    return {'on': by}
+        return {"left_on": list(by), "right_on": list(by.values())}
+    return {"on": by}
