@@ -1,16 +1,18 @@
 """Select helpers"""
 import re
 import builtins
-from typing import Callable, Iterable, List, Union
+from typing import Callable, List, Sequence, Union
 
-from pandas import DataFrame
+import numpy as np
+from pandas.api.types import is_scalar, is_bool
+from pandas.core.frame import DataFrame
 from pipda import register_func
 from pipda.utils import functype
 
 from ..core.contexts import Context
-from ..core.utils import get_option, vars_select, Array
-from ..core.types import StringOrIter
+from ..core.utils import ensure_nparray, vars_select, regcall
 from ..base import setdiff, intersect
+from .group_by import ungroup
 from .group_data import group_vars
 
 
@@ -27,25 +29,17 @@ def where(_data: DataFrame, fn: Callable) -> List[str]:
     Returns:
         The matched columns
     """
-    columns = everything(_data)
-    retcols = []
+    columns = regcall(everything, _data)
+    _data = regcall(ungroup, _data)
     pipda_type = functype(fn)
-    for col in columns:
-        if pipda_type == "plain":
-            conditions = fn(_data[col])
-        else:
-            conditions = fn(_data[col], __envdata=_data)
-
-        if isinstance(conditions, bool):
-            if conditions:
-                retcols.append(col)
-            else:
-                # pytest-cov not detecting this line
-                continue  # pragma: no cover
-        elif all(conditions):
-            retcols.append(col)
-
-    return retcols
+    mask = [
+        fn(_data[col])
+        if pipda_type == "plain"
+        else regcall(fn, _data[col], __envdata=_data)
+        for col in columns
+    ]
+    mask = [flag if is_bool(flag) else all(flag) for flag in mask]
+    return np.array(columns)[mask].tolist()
 
 
 @register_func(DataFrame)
@@ -58,14 +52,20 @@ def everything(_data: DataFrame) -> List[str]:
     Returns:
         All column names of _data
     """
-    return setdiff(_data.columns, group_vars(_data))
+    return list(
+        regcall(
+            setdiff,
+            _data.columns,
+            regcall(group_vars, _data),
+        )
+    )
 
 
 @register_func(context=Context.SELECT)
 def last_col(
     _data: DataFrame,
     offset: int = 0,
-    vars: Iterable[str] = None,  # pylint: disable=redefined-builtin
+    vars: Sequence[str] = None,
 ) -> str:
     """Select last variable, possibly with an offset.
 
@@ -86,9 +86,9 @@ def last_col(
 @register_func(context=Context.SELECT)
 def starts_with(
     _data: DataFrame,
-    match: StringOrIter,
+    match: Union[str, Sequence[str]],
     ignore_case: bool = True,
-    vars: Iterable[str] = None,  # pylint: disable=redefined-builtin
+    vars: Sequence[str] = None,
 ) -> List[str]:
     """Select columns starting with a prefix.
 
@@ -113,9 +113,9 @@ def starts_with(
 @register_func(context=Context.SELECT)
 def ends_with(
     _data: DataFrame,
-    match: str,
+    match: Union[str, Sequence[str]],
     ignore_case: bool = True,
-    vars: Iterable[str] = None,  # pylint: disable=redefined-builtin
+    vars: Sequence[str] = None,
 ) -> List[str]:
     """Select columns ending with a suffix.
 
@@ -142,7 +142,7 @@ def contains(
     _data: DataFrame,
     match: str,
     ignore_case: bool = True,
-    vars: Iterable[str] = None,  # pylint: disable=redefined-builtin
+    vars: Sequence[str] = None,
 ) -> List[str]:
     """Select columns containing substrings.
 
@@ -169,7 +169,7 @@ def matches(
     _data: DataFrame,
     match: str,
     ignore_case: bool = True,
-    vars: Iterable[str] = None,  # pylint: disable=redefined-builtin
+    vars: Sequence[str] = None,
 ) -> List[str]:
     """Select columns matching regular expressions.
 
@@ -194,8 +194,7 @@ def matches(
 @register_func(context=Context.EVAL)
 def all_of(
     _data: DataFrame,
-    x: Iterable[Union[int, str]],
-    base0_: bool = None,
+    x: Sequence[Union[int, str]],
 ) -> List[str]:
     """For strict selection.
 
@@ -205,8 +204,6 @@ def all_of(
     Args:
         _data: The data piped in
         x: A set of variables to match the columns
-        base0_: Whether `x` is 0-based or not.
-            if not provided, will use `datar.base.get_option('index.base.0')`
 
     Returns:
         The matched column names
@@ -216,7 +213,7 @@ def all_of(
             in `_data` columns
     """
     all_columns = _data.columns
-    x = all_columns[vars_select(all_columns, x, base0=base0_)]
+    x = all_columns[vars_select(all_columns, x)]
     # where do errors raise?
 
     # nonexists = setdiff(x, all_columns)
@@ -226,16 +223,14 @@ def all_of(
     #         f"Columns {nonexists} not exist."
     #     )
 
-    return list(x)
+    return x.tolist()
 
 
 @register_func(context=Context.SELECT)
 def any_of(
     _data: DataFrame,
-    x: Iterable[Union[int, str]],
-    # pylint: disable=redefined-builtin
-    vars: Iterable[str] = None,
-    base0_: bool = None,
+    x: Sequence[Union[int, str]],
+    vars: Sequence[str] = None,
 ) -> List[str]:
     """Select but doesn't check for missing variables.
 
@@ -245,14 +240,12 @@ def any_of(
     Args:
         _data: The data piped in
         x: A set of variables to match the columns
-        base0_: Whether `x` is 0-based or not.
-            if not provided, will use `datar.base.get_option('index.base.0')`
 
     Returns:
         The matched column names
     """
     vars = vars or _data.columns
-    x = vars_select(vars, x, raise_nonexists=False, base0=base0_)
+    x = vars_select(vars, x, raise_nonexists=False)
     # exists = []
     # for idx in x:
     #     try:
@@ -260,15 +253,20 @@ def any_of(
     #     except IndexError:
     #         ...
     # do we need intersect?
-    return intersect(vars, [vars[idx] for idx in x])
+    return list(
+        regcall(
+            intersect,
+            vars,
+            ensure_nparray(vars)[x],
+        )
+    )
 
 
 @register_func(None)
 def num_range(
     prefix: str,
-    range: Iterable[int],  # pylint: disable=redefined-builtin
+    range: Sequence[int],
     width: int = None,
-    base0_: bool = None,
 ) -> List[str]:
     """Matches a numerical range like x01, x02, x03.
 
@@ -278,23 +276,17 @@ def num_range(
         range_: A sequence of integers, like `range(3)` (produces `0,1,2`).
         width: Optionally, the "width" of the numeric range.
             For example, a range of 2 gives "01", a range of three "001", etc.
-        base0_: Whether it is 0-based
 
     Returns:
         A list of ranges with prefix.
     """
-    base0_ = get_option("index.base.0", base0_)
-    zfill = lambda elem: (
-        elem + int(not base0_)
-        if not width
-        else str(elem + int(not base0_)).zfill(width)
-    )
-    return Array([f"{prefix}{zfill(elem)}" for elem in builtins.range(range)])
+    zfill = lambda elem: (elem if not width else str(elem).zfill(width))
+    return [f"{prefix}{zfill(elem)}" for elem in builtins.range(range)]
 
 
 def _filter_columns(
-    all_columns: Iterable[str],
-    match: Union[Iterable[str], str],
+    all_columns: Sequence[str],
+    match: Union[Sequence[str], str],
     ignore_case: bool,
     func: Callable[[str, str], bool],
 ) -> List[str]:
@@ -309,11 +301,11 @@ def _filter_columns(
     Returns:
         A list of matched vars
     """
-    if not isinstance(match, (tuple, list, set)):
-        match = [match] # type: ignore
+    if is_scalar(match):
+        match = [match]  # type: ignore
 
     ret = []
-    for mat in match:
+    for mat in match:  # order kept this way
         for column in all_columns:
             if column in ret:
                 continue
